//Maemo Barcode Reader and Interpreter (mbarcode or maemo-barcode)
//Copyright (C) 2010 Simon Pickering
//Copyright (C) 2010 Svenn-Arne Dragly
//
//Some source code obtained from other individuals/companies (not affiliated with the project), such as
//Copyright (C) 2009 Nokia Corporation and/or its subsidiary(-ies).
//
//Various parts of barcode recognition and GStreamer manipulation code written by:
//      Timothy Terriberry
//      Adam Harwell
//      Jonas Hurrelmann
//
//Original GStreamer code based on the maemo-examples package:
//Copyright (C) 2007-2008 Nokia Corporation. All rights reserved.
//Copyright (C) 2006 INdT.
//@author Talita Menezes <talita.menezes@indt.org.br>
//@author Cidorvan Leite <cidorvan.leite@indt.org.br>
//@author Jami Pekkanen <jami.pekkanen@nokia.com>
//
//This program is free software: you can redistribute it and/or modify
//it under the terms of the GNU General Public License as published by
//the Free Software Foundation, either version 3 of the License, or
//(at your option) any later version.
//
//This program is distributed in the hope that it will be useful,
//but WITHOUT ANY WARRANTY; without even the implied warranty of
//MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//GNU General Public License for more details.
//
//You should have received a copy of the GNU General Public License
//along with this program.  If not, see <http://www.gnu.org/licenses/>.

#ifndef VIDEOWIDGET_H
#define VIDEOWIDGET_H
// qt includes
#include <QtOpenGL>
#include <QTime>
// gstreamer includes
#include <gst/gst.h>
#include <gst/gstbin.h>
#include <gst/interfaces/xoverlay.h>
#include <gst/interfaces/photography.h>
#include <gst/app/gstappsink.h>
// other includes
#include <stdio.h>
#include <zbar.h>
// local includes
#include "common.h"

class BarcodeDetector;
class DBusConnection;
class MaemoBarcodeWindow;
using namespace zbar;

class VideoWidget : public QGLWidget
{
    Q_OBJECT
public:
    VideoWidget(MaemoBarcodeWindow *parent);
    VideoWidget(MaemoBarcodeWindow *parent, BarcodeDetector *decoder);
    ~VideoWidget();
    void start(bool startEvenIfNotYetStarted = false);
    void stop(bool stopCompletely = false);
    bool isFocusing();
    void useVideosrc();
    void useFilesrc(QString filename);
    void setContinuousFocusTime(int milliseconds);
    bool isPipelineReady();

    // openGL functions
    static int buffer_width;
    static int buffer_height;
    bool cold_start;
    void refreshTexture();

    // refocus
    QTimer *refocusTimer;

    // timed scanning
    QTimer *timedScanTimer;
    void startTimedScan(int time);
    int timedScanInterval;

    void setContinuousScanning(bool on);


public slots:
    void refocus();
    void stopTimedScan();


signals:
    void focusingValueChanged(bool enabled);
    void doSaveBuffer();

protected:
    void resizeGL(int w, int h);
    void paintGL();
    void initializeGL();

private:
    void setFocusing(bool enabled);
    bool keepRunning;
    // openGL functions
    void paintTexturedCube();
    void paintQtLogo();
    void refreshBuffer();

    void newBufferAdded (GstAppSink *_appsink, gpointer user_data);
    static void new_buffer_added_callback (GstAppSink *_appsink, gpointer user_data);

    // Gst functions
    gboolean busCallback (GstBus * bus, GstMessage * message);
    static gboolean busCallback_callback (GstBus * bus, GstMessage * message);
    gboolean bufferProbeCallback (char *data);
//    static gboolean bufferProbeCallback_callback (GstElement *image_sink, GstBuffer *buffer, GstPad *pad);
    void destroyPipeline();
    bool initPipeline();
    void getDeviceType();

    bool pipelineReady;
    BarcodeDetector *decoder;
    GstElement *pipeline;

    // added from appdata.h
    GstElement *camera_src;
    GstElement *file_src;
    GstElement *input_selector;
    GstElement *video_crop;

    gboolean isportrait;
    gboolean accelerometers;
    DBusConnection *con;
    guint buffer_cb_id;
    guint timer_id;
    long long last_upc_value;
    int scan_in_progress;
    gchar current_data_table_name[255];
    int deviceType;
    int frame_width;
    int frame_height;
    zbar_image_scanner_t *scanner;
    zbar_image_t *image;
    bool focusing;

    // openGL painting
    int frames;
    //QTime frametime;
    QTimer *timerOpenGL;
    GLuint m_uiTexture;
    QGLShaderProgram *program2;
//    GLuint program;
    int vertexAttr;
    int normalAttr;
    int texCoordAttr;
    int matrixUniform;
    int textureUniform;
    bool qtLogo;
    bool m_showBubbles;
    bool m_showBackground;
    QTime time;
    GstBuffer *gstBuffer;
    bool buffer_ready;
    GstAppSink *appsink;
    bool appSinkScanning;


    // GUI stuff
    QImage imgExit;
    QPoint imgExitPos;
    QImage imgSave;
    QPoint imgSavePos;
    QImage imgTaskManager;
    QPoint imgTaskManagerPos;
    QImage imgRecent;
    QPoint imgRecentPos;
    QImage imgSettings;
    QPoint imgSettingsPos;
    QImage imgOpen;
    QPoint imgOpenPos;

    int continuousFocusInterval;

    //
    MaemoBarcodeWindow *parent;



private slots:
    void mousePressEvent(QMouseEvent *event);
    void setContinuousScanningCallback();
};

#endif // VIDEOWIDGET_H
